//! Routing integration tests

pub mod cache_aware_backward_compat_test;
pub mod header_forwarding_test;
pub mod load_balancing_test;
pub mod manual_routing_test;
pub mod payload_size_test;
pub mod pd_routing_test;
pub mod policy_registry_integration;
pub mod power_of_two_test;
pub mod service_discovery_test;
pub mod test_openai_routing;
pub mod test_pd_routing;
pub mod worker_management_test;
